//SETA43 19/06/2024
// seta43.duckdns.org
// seta43.blogspot.com /
// reproductor de wav desde tarjeta MEMORIA
// Comando serie
// wav 11kHz mono
// Arduino nano


#include <stdint.h>
#include <SPI.h>


struct ESTA {
    char numero;
    char nombre[12];
    long ini;
    unsigned int tama;
};

struct ESTA estacion[]={ {0,"nada", 0,0},
        {1,"Vbeep", 0,25960},
        {2,"Vproxima",28017,15983},
        {3,"Vcuatro", 44869,13409},
        {4,"Vrios", 59477,14443},
        {5,"Viglesia", 74000,13153},
        {6,"Vbilbao" , 86636,12716},
        {7,"Vtribunal", 98670,12330},
        {8,"Vgran", 112244,15477},
        {9,"Vsol" , 128238,10824},
        {10,"Vsol",128238,10824} };

int nEstaciones=sizeof(estacion)/sizeof(*estacion);

unsigned char miCola[10];

char putCola(unsigned char n)
{
 int z;
    for(z=0;z<10;z++)
    {
        if(miCola[z]==0) {miCola[z]=n; break;}
    }
    return(z);
}

char getCola()
{
 char c=miCola[0];
    for(int z=0;z<9;z++)  miCola[z]=miCola[z+1];
    miCola[9]=0;
 return(c);
}


//---Pin CS Memoria ---
#define PIN_CS  10
// -Almacen lectura SPI
#define PAGE_SIZE 2
uint8_t data[PAGE_SIZE];

//--Variable buffer de reproducción
byte valor;
byte bufer0[100];
byte bufer1[100];
int din=0;
int dout=0;
byte nf=0;
byte vacio=1;
byte fin=1;

//---Pin Salida audio ---
#define AUDIO 9

//--buffer entrada puerto serie
#define MAXENTRADA 15
char buffer[MAXENTRADA];
int contador=0;
char stringComplete = false;

uint32_t dire,tama;


void setup() 
{
  Serial.begin(9600);
  while (!Serial);
  Serial.print("N.Estaciones=");
  Serial.println(nEstaciones);

  pinMode(PIN_CS, OUTPUT);
  pinMode(AUDIO, OUTPUT);

  SPI.begin();
  SPI.beginTransaction(SPISettings(8000000, MSBFIRST, SPI_MODE0));
  reset();
  readId();

  cli(); //stop interrupts
  TCCR1A = 0x81;
  TCCR1B = 0x01;
  OCR1A = 255;
  TIMSK1 = 0;

  //timer interrupts
  //by Amanda Ghassaei
  //set timer2 interrupt at 11kHz
  TCCR2A = 0; // set entire TCCR2A register to 0
  TCCR2B = 0; // same for TCCR2B
  TCNT2 = 0; //initialize counter value to 0
  OCR2A = 182; // = (16*10^6) / (11000*8) - 1 (must be <256)  11khz
  // turn on CTC mode
  TCCR2A |= (1 << WGM21);
  // Set CS21 bit for 8 prescaler
  TCCR2B |= (1 << CS21);
  // enable timer compare interrupt
  TIMSK2 |= (1 << OCIE2A);
  sei(); //allow interrupts
}



void loop()
{
  //----Tratamiento entrada teclado ----
  while (Serial.available())
        {
          char inChar = (char)Serial.read(); 
          buffer[contador]=inChar;  
          if (inChar == '\n')
              {
                stringComplete = true;
                buffer[contador]=0;
              }       
          if(contador<MAXENTRADA)contador++;
        }
      if (stringComplete) 
        {     
          runFuniones();
          contador=0;
          stringComplete = false;   
        }

//----Recarga buffer audio ----
  if(vacio == 1 && fin == 0)
    {     
     if(nf==0) for(din=0;din<100;din++) bufer0[din] = readMemoria();
     if(nf==1) for(din=0;din<100;din++) bufer1[din] = readMemoria();
     vacio=0;
     if(dire>tama)
       {
         for(din=0;din<100;din++){ bufer0[din]=128;  bufer1[din]=128;} //vacia bufer
         fin=1;
       }
    }

//-----Reproduce cola de audio -----
 if(fin==1) colaPlay();    
 
}
//--------------------------------------------------------------






//---Vector de interrupción-----
ISR(TIMER2_COMPA_vect)
{
  //timer1 interrupt 11kHz toggles
  if(nf==0) valor=bufer1[dout];
  if(nf==1) valor=bufer0[dout];
  dout++;
  if(dout>99)
    {
     vacio=1;
     dout=0;
     if(nf==1)
       nf=0;
      else
       nf=1;       
    }  
  OCR1AH = 0;
  OCR1AL = valor;     
}



char readMemoria(void)
{
  readData(dire++,data,1);
  if(dire>tama) data[0]=128;
  return(data[0]);
}


char playWav(uint32_t di,uint32_t ta)
{
  dire=di;
  tama=dire+ta;
  fin=0;
  return(0);
}



//----------------------------------------------
void colaPlay()
{
  int n;
  n=getCola();
  if(n!=0)   playWav(estacion[n].ini,estacion[n].tama);  
}


int runFuniones(void)
{
  int numero,en;
      numero=atoi(buffer);
      if(numero==0)
        {       
          Serial.println("Stop cancion");
          for(din=0;din<100;din++){ bufer0[din]=128;  bufer1[din]=128;} //vacia bufer
          dire=0;
          fin=1;          
          goto salir0;
        }
      if(numero>nEstaciones) {Serial.println("ERROR N.estacion"); goto salir0;}
      for(en=0;en<nEstaciones;en++) 
        {
        if(estacion[en].numero==numero) 
          {
            Serial.print(en);Serial.print(" Estación="); Serial.println(estacion[en].nombre);
            putCola(1); putCola(2);  putCola(en);
            goto salir0;
          }
        }

      Serial.println("aERROR ");
salir0:
 ;
}      




//-------------SPI_MEMORY--------https://hackaday.io/page/6060-spi-flash-test----------------

void sendOneByteCommand(uint8_t command) {
  digitalWrite(PIN_CS, LOW);
  SPI.transfer(command);
  digitalWrite(PIN_CS, HIGH);
}

uint8_t readStatusRegister() {
  digitalWrite(PIN_CS, LOW);
  SPI.transfer(0x05);
  uint8_t result = SPI.transfer(0);
  digitalWrite(PIN_CS, HIGH);
  return result;
}

void reset() {
  sendOneByteCommand(0x66);
  sendOneByteCommand(0x99);
  delay(20);
}

void readId() {
  digitalWrite(PIN_CS, LOW);
  SPI.transfer(0x9f);
  uint8_t manufacturerId = SPI.transfer(0);
  uint8_t memoryType = SPI.transfer(0);
  uint8_t memoryDensity = SPI.transfer(0);
  digitalWrite(PIN_CS, HIGH);

  Serial.print("manufacturer ID: 0x");
  Serial.print(manufacturerId, HEX);
  Serial.print(", memory type: 0x");
  Serial.print(memoryType, HEX);
  Serial.print(", memory density: 0x");
  Serial.println(memoryDensity, HEX);
}

void writeAddress(uint32_t address) 
{
  SPI.transfer((address >> 16) & 0xff);
  SPI.transfer((address >> 8) & 0xff);
  SPI.transfer(address & 0xff);
}

// reads data from flash
unsigned long readData(uint32_t address, uint8_t* data, uint32_t size) {
  digitalWrite(PIN_CS, LOW);
  SPI.transfer(0x03);
  writeAddress(address);
  SPI.transfer(data, size);
  digitalWrite(PIN_CS, HIGH);
}
//------------------------------------------------------------
